/*
 * Implement for HTML Document object's interfaces:
 *   IHTMLDocument
 *
 * Copyright (c) 2004 TransGaming Technologies
 */

#include "winbase.h"
#include "winuser.h"
#include "wingdi.h"

#include "oleauto.h"
#include "htmlguid.h"
#include "mshtml.h"

#include "wine/debug.h"
#include "wine/unicode.h"

#include "mshtml_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(mshtml);

static ICOM_VTABLE(IDispatch) mshtmlDispatchVT;
static ICOM_VTABLE(IHTMLDocument) mshtmlHTMLDocumentVT;
static ICOM_VTABLE(IHTMLDocument2) mshtmlHTMLDocument2VT;

/* IDispatch */

static HRESULT WINAPI html_Dispatch_QueryInterface(LPDISPATCH iface, REFIID riid,
                                                   LPVOID *ppobj)
{
    ICOM_THIS(HTMLDocumentImpl, iface);

    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), ppobj);

    if (IsEqualGUID(&IID_IUnknown, riid) ||
        IsEqualGUID(&IID_IDispatch, riid))
    {
        TRACE("IUnknown or IDispatch\n");
        This->ref++;
        *ppobj = This;
        return S_OK;
    }
    else if (IsEqualGUID(&IID_IHTMLDocument, riid))
    {
        TRACE("IHTMLDocument\n");
        This->ref++;
        *ppobj = (LPVOID *)&This->lpVtbl_HTMLDocument;
        return S_OK;
    }
    else if (IsEqualGUID(&IID_IHTMLDocument2, riid))
    {
        TRACE("IHTMLDocument2\n");
        This->ref++;
        *ppobj = (LPVOID *)&This->lpVtbl_HTMLDocument2;
        return S_OK;
    }
    else
    {
        FIXME("(%p)->(%s, %p): no interface\n", iface, debugstr_guid(riid), ppobj);
        return E_NOINTERFACE;
    }
}

static ULONG WINAPI html_Dispatch_AddRef(LPDISPATCH iface)
{
    ICOM_THIS(HTMLDocumentImpl, iface);
    TRACE("(%p)->()\n", iface);
    return ++(This->ref);
}

static ULONG WINAPI html_Dispatch_Release(LPDISPATCH iface)
{
    ICOM_THIS(HTMLDocumentImpl, iface);
    TRACE("(%p)->()\n", iface);

    if (--(This->ref)) return This->ref;

    HeapFree(GetProcessHeap(), 0, This);
    return 0;
}

/* IHTMLDocument */

static HRESULT WINAPI html_HTMLDocument_QueryInterface(LPHTMLDOCUMENT iface,
                                                       REFIID riid, LPVOID *ppobj)
{
    ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument, iface);
    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), ppobj);
    return IDispatch_QueryInterface((LPDISPATCH)This, riid, ppobj);
}

static ULONG WINAPI html_HTMLDocument_AddRef(LPHTMLDOCUMENT iface)
{
    ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument, iface);
    TRACE("(%p)\n", iface);
    return IDispatch_AddRef((LPDISPATCH)This);
}

static ULONG WINAPI html_HTMLDocument_Release(LPHTMLDOCUMENT iface)
{
    ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument, iface);
    TRACE("(%p)\n", iface);
    return IDispatch_Release((LPDISPATCH)This);
}

/* IHTMLDocument2 */

static HRESULT WINAPI html_HTMLDocument2_QueryInterface(LPHTMLDOCUMENT2 iface,
                                                        REFIID riid, LPVOID *ppobj)
{
    ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument2, iface);
    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), ppobj);
    return IDispatch_QueryInterface((LPDISPATCH)This, riid, ppobj);
}

static ULONG WINAPI html_HTMLDocument2_AddRef(LPHTMLDOCUMENT2 iface)
{
    ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument2, iface);
    TRACE("(%p)\n", iface);
    return IDispatch_AddRef((LPDISPATCH)This);
}

static ULONG WINAPI html_HTMLDocument2_Release(LPHTMLDOCUMENT2 iface)
{
    ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument2, iface);
    TRACE("(%p)\n", iface);
    return IDispatch_Release((LPDISPATCH)This);
}

static HRESULT WINAPI html_HTMLDocument2_write(LPHTMLDOCUMENT2 iface,
                                               SAFEARRAY* psarray)
{
    /*ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument2, iface);*/
    FIXME("(%p)->(%p): stub\n", iface, psarray);
    return S_OK;
}

static HRESULT WINAPI html_HTMLDocument2_close(LPHTMLDOCUMENT2 iface)
{
    /*ICOM_THIS_MULTI(HTMLDocumentImpl, lpVtbl_HTMLDocument2, iface);*/
    FIXME("(%p): stub\n", iface);
    return S_OK;
}

/* other stuff */

HRESULT html_CreateDocument(IUnknown *pOuter, REFIID riid, LPVOID *ppobj)
{
    HTMLDocumentImpl *document;
    HRESULT hr;

    TRACE("()\n");

    document = (HTMLDocumentImpl*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
                                            sizeof(HTMLDocumentImpl));
    if (document == NULL) return E_OUTOFMEMORY;

    ICOM_VTBL(document) = &mshtmlDispatchVT;
    document->lpVtbl_HTMLDocument = &mshtmlHTMLDocumentVT;
    document->lpVtbl_HTMLDocument2 = &mshtmlHTMLDocument2VT;

    IDispatch_AddRef((LPDISPATCH)document);
    hr = IDispatch_QueryInterface((LPDISPATCH)document, riid, ppobj);
    IDispatch_Release((LPDISPATCH)document);

    if (SUCCEEDED(hr))
        return S_OK;
    else
        return E_NOINTERFACE;
}

static ICOM_VTABLE(IDispatch) mshtmlDispatchVT =
{
    ICOM_MSVTABLE_COMPAT_DummyRTTIVALUE
    /* IUnknown */
    html_Dispatch_QueryInterface,
    html_Dispatch_AddRef,
    html_Dispatch_Release,
    /* IDispatch */
    0xdeada000, /* GetTypeInfoCount */
    0xdeada001, /* GetTypeInfo */
    0xdeada002, /* GetIDsOfName */
    0xdeada003, /* Invoke */
};

static ICOM_VTABLE(IHTMLDocument) mshtmlHTMLDocumentVT =
{
    ICOM_MSVTABLE_COMPAT_DummyRTTIVALUE
    /* IUnknown */
    html_HTMLDocument_QueryInterface,
    html_HTMLDocument_AddRef,
    html_HTMLDocument_Release,
    /* IDispatch */
    0xdeada100, /* GetTypeInfoCount */
    0xdeada101, /* GetTypeInfo */
    0xdeada102, /* GetIDsOfName */
    0xdeada103, /* Invoke */
    /* IHTMLDocument */
    0xdeada104  /* get_Script */
};

static ICOM_VTABLE(IHTMLDocument2) mshtmlHTMLDocument2VT =
{
    ICOM_MSVTABLE_COMPAT_DummyRTTIVALUE
    /* IUnknown */
    html_HTMLDocument2_QueryInterface,
    html_HTMLDocument2_AddRef,
    html_HTMLDocument2_Release,
    /* IDispatch */
    0xdeada200, /* GetTypeInfoCount */
    0xdeada201, /* GetTypeInfo */
    0xdeada202, /* GetIDsOfName */
    0xdeada203, /* Invoke */
    /* IHTMLDocument */
    0xdeada204,  /* get_Script */
    /* IHTMLDocument2 */
    0xdeada205, /* get_all */
    0xdeada206, /* get_body */
    0xdeada207, /* get_activeElement */
    0xdeada208, /* get_images */
    0xdeada209, /* get_applets */
    0xdeada20a, /* get_links */
    0xdeada20b, /* get_forms */
    0xdeada20c, /* get_anchors */
    0xdeada20d, /* put_title */
    0xdeada20e, /* get_title */
    0xdeada20f, /* get_scripts */
    0xdeada210, /* put_designMode */
    0xdeada211, /* get_designMode */
    0xdeada212, /* get_selection */
    0xdeada213, /* get_readyState */
    0xdeada214, /* get_frames */
    0xdeada215, /* get_embeds */
    0xdeada216, /* get_plugins */
    0xdeada217, /* put_alinkColor */
    0xdeada218, /* get_alinkColor */
    0xdeada219, /* put_bgColor */
    0xdeada21a, /* get_bgColor */
    0xdeada21b, /* put_fgColor */
    0xdeada21c, /* get_fgColor */
    0xdeada21d, /* put_linkColor */
    0xdeada21e, /* get_linkColor */
    0xdeada21f, /* put_vlinkColor */
    0xdeada221, /* get_vlinkColor */
    0xdeada222, /* get_referrer */
    0xdeada223, /* get_location */
    0xdeada224, /* get_lastModified */
    0xdeada225, /* put_URL */
    0xdeada226, /* get_URL */
    0xdeada227, /* put_domain */
    0xdeada228, /* get_domain */
    0xdeada229, /* put_cookie */
    0xdeada22a, /* get_cookie */
    0xdeada22b, /* put_expando */
    0xdeada22c, /* get_expando */
    0xdeada22d, /* put_charset */
    0xdeada22e, /* get_charset */
    0xdeada22f, /* put_defaultCharset */
    0xdeada231, /* get_defaultCharset */
    0xdeada232, /* get_mimeType */
    0xdeada233, /* get_fileSize */
    0xdeada234, /* get_fileCreatedDate */
    0xdeada235, /* get_fileModifiedDate */
    0xdeada236, /* get_fileUpdatedDate */
    0xdeada237, /* get_security */
    0xdeada238, /* get_protocol */
    0xdeada239, /* get_nameProp */
    html_HTMLDocument2_write,
    0xdeada23b, /* writeln */
    0xdeada23c, /* open */
    html_HTMLDocument2_close,
    0xdeada23e, /* clear */
    0xdeada23f, /* queryCommandSupported */
    0xdeada241, /* queryCommandEnabled */
    0xdeada242, /* queryCommandState */
    0xdeada243, /* queryCommandIndeterm */
    0xdeada244, /* queryCommandText */
    0xdeada245, /* queryCommandValue */
    0xdeada246, /* execCommand */
    0xdeada247, /* execCommandShowHelp */
    0xdeada248, /* createElement */
    0xdeada249, /* put_onhelp */
    0xdeada24a, /* get_onhelp */
    0xdeada24b, /* put_onclick */
    0xdeada24c, /* get_onclick */
    0xdeada24d, /* put_ondblclick */
    0xdeada24e, /* get_ondblclick */
    0xdeada24f, /* put_onkeyup */
    0xdeada251, /* get_onkeyup */
    0xdeada252, /* put_onkeydown */
    0xdeada253, /* get_onkeydown */
    0xdeada254, /* put_onkeypress */
    0xdeada255, /* get_onkeypress */
    0xdeada256, /* put_onmouseup */
    0xdeada257, /* get_onmouseup */
    0xdeada258, /* put_onmousedown */
    0xdeada259, /* get_onmousedown */
    0xdeada25a, /* put_onmousemove */
    0xdeada25b, /* get_onmousemove */
    0xdeada25c, /* put_onmouseout */
    0xdeada25d, /* get_onmouseout */
    0xdeada25e, /* put_onmouseover */
    0xdeada25f, /* get_onmouseover */
    0xdeada261, /* put_onreadystatechange */
    0xdeada262, /* get_onreadystatechange */
    0xdeada263, /* put_onafterupdate */
    0xdeada264, /* get_onafterupdate */
    0xdeada265, /* put_onrowexit */
    0xdeada266, /* get_onrowexit */
    0xdeada267, /* put_onrowenter */
    0xdeada268, /* get_onrowenter */
    0xdeada269, /* put_ondragstart */
    0xdeada26a, /* get_ondragstart */
    0xdeada26b, /* put_onselectstart */
    0xdeada26c, /* get_onselectstart */
    0xdeada26d, /* elementFromPoint */
    0xdeada26e, /* get_parentWindow */
    0xdeada26f, /* get_styleSheets */
    0xdeada271, /* put_onbeforeupdate */
    0xdeada272, /* get_onbeforeupdate */
    0xdeada273, /* put_onerrorupdate */
    0xdeada274, /* get_onerrorupdate */
    0xdeada275, /* toString */
    0xdeada276  /* createStyleSheet */
};

